/*

Directional Grid library by ACWraith

////////////////////////////////////////////////////////////////////////////////
LICENSE:	You may use this library for profit or non-profit.
			Credit is appreciated, but not required.

	If you need a more formal license, consider it to be:
		Creative Commons Attribution 3.0 Unported (CC BY 3.0)
		http://creativecommons.org/licenses/by/3.0/

	CC licenses generally are't recommended for source code.
	However, I figure I can keep the rules simple by using the
	same license as any other resources I may make open.  (For
	instance, Fooldom Come subscribers get free reign over the sound
	and	graphics from some of my games.)  Contact me if you need a
	more specialized license:  ACWraith@gmail.com

////////////////////////////////////////////////////////////////////////////////

DESCRIPTION:

	This library stores up to eight directions in each cell of a two dimensional
	grid.  It can be used to generate mazes both manually and automatically.

	Differences from my Prim Grid library include:
		*	DG offers a direct grid2d_Grid derivative with a cleaner interface.
		*	Edge wrapping can be toggled at will instead of on creation/reset.
		*	Connections can be made manually with coordinates and/or directions
			rather than bits or hidden indices.
		*	Prim's algorithm is still provided, but whether or not the
			connections reciprocate can be determined within each the proc call.
		*	Various cell categories can be identified and gathered.

////////////////////////////////////////////////////////////////////////////////
dg_Grid

	A dg_Grid is a derivative of grid2d_Grid.  All values are stored internally.
	Creating and reseting does not require a default value.

		New(Width=1,Height=1)
			If Width is a dg_Grid, New() becomes a copy constructor and Height
			is ignored.  Other grid2D_Grid types are possible as long as they
			hold only numbers.
		grid2d_CopyIntoSelf(grid2d_Grid/Grid)
			Copys Grid into the dg_Grid.  Grid can be a dg_Grid or any
			grid2d_Grid of nums.
		grid2d_Reset(Width=1,Height=1)
			Sets the grid dimensions to Width by Height.  If Width is a dg_Grid
			or any grid2d_Grid of nums, its values will be used and the other
			arguments will be ignored.
			Returns:
				0 on failure
				negative if the result is a copy.
				positive if the result is not a copy



	//===========================================================================
	CONNECTIONS

	Connections can be one-way rather than reciprocating.  When connecting cells
	manually, know that each cell stores what neighbors it exits into.  Setting
	an entrance actually sets the exit of a neighbor.  (Note that grid2d_Grid's
	neighbor direction restrictions are ignored.  These procs simply obey.)

		dg_Isolate(X,Y)
			Removes all connections coming in and out of cell X,Y.
			Returns TRUE if the event occured.

		dg_SetEntranceDir(On=TRUE,X,Y,Dir)
			If On, this creates an entrance to FromX,FromY from the cell in
			direction Dir.  Otherwise, such an entrance is removed.

		dg_SetExitCell(On=TRUE,FromX,FromY,ToX,ToY)
			If On, this creates an exit for cell FromX,FromY to cell ToX,ToY.
			Otherwise, such an exit is removed.

		dg_SetExitDir(On=TRUE,X,Y,Dir)
			If On, this creates an exit for cell X,Y in direction Dir.
			Otherwise, such an exit is removed.



	//===========================================================================
	ALGORITHMS

	Of course, connections can be automated as well.  One algorithm is provided.
	When creating your own, it might be wise to deal with indices internally
	rather than	coordinates	so that cells can be identified by a single key.
	Grid 2D may provide some useful information to maze creators in index.dm.

		dg_Prim(StartX,StartY,ReciprocateExits=FALSE)
			Draws a maze within the grid using Prim's algorithm.  If StartX and
			StartY are not valid coordinates, a random start cell will be chosen.
			If ReciprocateExits, connections between cells will go both ways.

			Note that previous grid information will not be cleared first.
			If desired, that should be done manually with grid2d_Reset().



	In addition, cells can be excluded from the next maze generated.  (This works
	by filtering them out of Grid 2D's neighbor procs.)  Some algorithms, like
	Prim's, will fill whatever shape is available.
		Tip:	dg_Prim() makes sure the start coordinates are included.  Perhaps
				your algorithm should too.

		dg_AddExclusion(X,Y,EnsureUnique=FALSE)
			Adds the coordinates X,Y to the exclusion list.  Setting EnsureUnique
			to true will avoid the risk of leaving copies behind when removing,
			but it makes this slower with a large number of exclusions.
			Returns
				0 on failure
				-1 if it was already in there and EnsureUnique is true
				the grid index added (always positive)

		dg_ClearExclusions()
			Removes all exclusions and returns the list of indices formally excluded.
			This is much faster than removing coordinates individually.

		dg_GetExcluded()
			Returns a list containing the coordinates excluded.
		dg_GetIncluded()
			Returns a list containing the coordinates which are not excluded.

		dg_IsExcluded(X,Y)
			Returns whether or not the coordinates X,Y are excluded from the maze.

		dg_RemoveExclusion(X,Y,RemoveAll=FALSE)
			Removes the coordinates X,Y from the exclusion list.  Setting RemoveAll
			to true will avoid leaving copies behind, but it's slower.
			Returns
				0 on failure
				the amount of removals if RemoveAll is true
				the grid index removed if RemoveAll is false


	Advanced users creating their own algorithms might find the following two procs
	useful.

		dg_GetExcludedIndices(Copy=TRUE)
			Returns a list containing the grid indices excluded.  If Copy is false
			this is the actual internal storage.  (Don't edit it directly!)
		dg_GetIncludedIndices()
			Returns a list containing the indices which are not excluded.


	//===========================================================================
	EXITS AND ENTRANCES

	Various procs are included for reading exits.

		dg_DoesExitToCell(FromX,FromY,ToX,ToY)
			Returns whether or not the cell at FromX,FromY exits into the cell at ToX,ToY.

		dg_DoesExitToDir(X,Y,Dir)
			Returns whether or not the cell at X,Y has an exit in direction Dir.

		dg_GetExitDirs(X,Y)
			Returns a list of the directions exiting from the cell at X,Y.

		dg_GetExits(X,Y)
			Returns a list of coordinates where the exits of cell X,Y lead to.

		dg_GetNumExits(X,Y)
			Returns how many exits the cell at X,Y has.

		dg_HasExits(X,Y)
			Returns whether or not the cell has any exits at all.



	Likewise, various procs are also included for reading entrances.

		dg_GetEntranceDirs(X,Y)
			Returns a list of directions from the cell at X,Y to each neighboring
			cell that enters it.

		dg_GetEntrances(X,Y)
			Returns a list of coordinates of cells with entrances to cell X,Y.

		dg_GetNumEntrances(X,Y)
			Returns how many neighboring cells enter into the cell at X,Y.

		dg_HasEntrances(X,Y)
			Returns whether or not the cell has any entrances at all.

		dg_IsEnteredByCell(FromX,FromY,ToX,ToY)
			Returns whether or not the cell at FromX,FromY is entered by the cell at ToX,ToY.

		dg_IsEnteredByDir(X,Y,Dir)
			Returns whether or not the cell in direction Dir of X,Y enters X,Y.
			Ex: Does the cell to the north have a southern exit?



	//===========================================================================
	CELL TYPES

	Cells can be categorized based on their exits and entrances.  Some cells
	might belong in multiple categories depending on whether connections are
	reciprocating.  Procs are included for listing cells of specified categories
	and determining whether a specified cell

		A DEAD END has one and only one entrance.  If it has an exit, it exits
		into the cell which enters it.
			dg_GetDeadEnds()
			dg_IsDeadEnd(X,Y)

		An ORIGIN has one and only one exit.  If it has an entrance, the cell
		entering it is the one that the exit leads to.
			dg_GetOrigins()
			dg_IsOrigin(X,Y)

		A TUNNEL allows travel between exactly two nodes in at least one direction.
			dg_GetTunnels()
			dg_IsTunnel(X,Y)

		A FORK has multiple exits and at least one entrance.
			dg_GetForks()
			dg_IsFork(X,Y)

		A FUNNEL has multiple entrances and at least one exit.
			dg_GetFunnels()
			dg_IsFunnel(X,Y)

		An ISOLATED cell has no exits or entrances
			dg_GetIsolated()
			dg_IsIsolated(X,Y)



	//===========================================================================
	PATHFINDING

	Procs are included for finding the distance between cells through the maze.

		dg_GetEntranceDistance(TargetX,TargetY,EnteredX,EnteredY)
			If Entered is entered by Target (directly or indirectly), this
			returns the minimum number of entrances between them.
			Returns -1 on failure.

		dg_GetExitDistance(ExitedX,ExitedY,TargetX,TargetY)
			If Exited exits to Target (directly or indirectly), this returns the
			minimum number of exits between them.  Returns -1 on failure.


	If making multiple calls based around the same pair of coordinates, a
	dg_DistanceGrid might be more efficient.  Its New() proc should be considered
	private.  Use the following proc from dg_Grid instead:

		dg_GetDistanceGrid(StartX,StartY,ExitsNotEntrances=TRUE,EndX=null,EndY=null)
			Returns a dg_DistanceGrid with StartX,StartY as its center.  If
			ExitsNotEntrances, the distances are based on exits.  Otherwise,
			they're based on entrances.  Returns null on failure.

			By default, this records the distance to every node accessible from
			StartX,StartY.  If EndX and EndY are not null, the grid will stop
			recording distances when/if their cell is reached.  Remember to check
			the	grid's dg_GetEndX() and dg_GetEndY() procs to verify that
			EndX,EndY was actually accessible.

	Once created, a cell's distance from the start cell is simply its value in the
	dg_DistanceGrid.  That grid type also offers the following:

		dg_DistanceGrid

			dg_GetEndX()
				Returns the X coordinate of the last cell checked...  if defined!
			dg_GetEndY()
				Returns the Y coordinate of the last cell checked...  if defined!

			dg_GetMaxDistance()
				Returns the maximum distance stored in the table.

			dg_GetStartX()
				Returns the X coordinate of the cell the distances are from.
			dg_GetStartY()
				Returns the Y coordinate of the cell the distances are from.

			dg_IsExits()
				If true, this is storing exit distances.  Otherwise, it's storing
				entrance distances.
*/