/*

	The following is more of an interactive demo than a programming one.
	The grid is still an abstract structure.  To give it form, this demo
	considers each cell to be a turf.  You don't have to.

	There are 4900 of those turfs.  That might be more than slow computers
	would wish to handle.  The number was chosen merely because it's the
	maximum quantity of turfs which can be kept in view at the time of this
	writing.  You may wish to decrease the map dimensions.

*/
world
	maxx	= 70
	maxy	= 70



client
	/*
		We need to create a dg_Grid somewhere.  I'm choosing to stuff
		everything under client for this demo, but your project will likely
		have different requirements.
	*/
	var/dg_Grid/myGrid
	New()
		..()
		src.myGrid	= new(world.maxx,world.maxy)
		//	We'll avoid diagonal neighbors because the connections are harder to display.
		src.myGrid.grid2d_SetNeighborDirs(list(NORTH,EAST,SOUTH,WEST))


	/*
		Prim's algorithm is included for maze generation.  If implementing
		another algorithm, you may wish to refer to Grid 2D's unlisted index
		procs.  Either way, exits can still be toggled manually.
	*/
	verb/Generate_Maze()
		//	Note that the maze will be merged into what connections are
		//	already there.  Let's first clear the grid.
		src.myGrid.grid2d_Reset(world.maxx,world.maxy)
		//	Setting ReciprocateConnections to TRUE will make sure cells exit
		//	into cells which enter them.  Otherwise, only exits will be set.
		src.myGrid.dg_Prim(rand(1,world.maxx),rand(1,world.maxy),TRUE)
		//	The data structure is abstract.  Let's display it somehow.
		src.Display()
		world	<< ">> New maze generated.  Turfs can be clicked for info."


	/*
		For this demo, let's make a visual representation of the maze by
		highlighting the cell categories.
	*/
	proc/Display()
		var
			area
				Isolated/isolated	= new
				DeadEnd/deadEnds	= new
				Fork/forks			= new
				Tunnel/tunnels		= new
			turf/T
		for( T in  block(locate(1,1,1),locate(world.maxx,world.maxy,1)) )
			//	Displaying the walls is easy if using icons created by Lummox JR's
			//	IconCutter.  The internal direction flags are the same.
			T.icon_state	= "[src.myGrid.grid2d_GetItem(T.x,T.y)]"
			//	We'll isolate some nodes later.
			if ( src.myGrid.dg_IsIsolated(T.x,T.y) )		isolated.contents	+= T
			//	In a reciprocating maze, all forks are also funnels.
			//	We don't need the extra check unless we set exits manually.
			else if ( src.myGrid.dg_IsFork(T.x,T.y) )		forks.contents		+= T
			//	Tunnels are tunnels whether they go both ways or not.
			else if ( src.myGrid.dg_IsTunnel(T.x,T.y) )		tunnels.contents	+= T
			//	Origins are the same as dead ends when reciprocating.
			else if ( src.myGrid.dg_IsDeadEnd(T.x,T.y) )	deadEnds.contents	+= T


	/*
		If we already know a cell doesn't block our path, we can simply isolate it.
		This can help adjust the number of rooms and shape the maze into something
		other than a rectangle.

		In a more open map, the random walls formed by dead ends might be the
		only thing we're interested in.  The short turns of Prim's algorithm
		offer plenty and we know that the rest of the maze guarantees a route
		around them.
	*/
	verb/Isolate_Dead_Ends()
		//	Cell lists are returned as lists of coordinates.  I find it handy to
		//	start at half the list length and work backwards.
		var
			list/deadEnds	= src.myGrid.dg_GetDeadEnds()
			index			= deadEnds.len/2
		while(index>0)
			src.myGrid.dg_Isolate(deadEnds[2*index-1],deadEnds[2*index])
			index--
		//	Note that some mazes might be cut down to one cell surrounded
		//	by two dead ends.  When those dead ends are isolated, that cell
		//	will be as well even though dg_Isolate() wasn't called with its
		//	coordinates.
		src.Display()
		world	<< ">> Dead ends isolated."


	/*
		Connections can extend through edges if wrapping is on.  Note that you
		won't see a difference until checking for neighbors again.
	*/
	verb/Toggle_Wrap_X()
		if ( src.myGrid.grid2d_DoesWrapX() )
			src.myGrid.grid2d_SetWrap(FALSE,src.myGrid.grid2d_DoesWrapY())
			world	<< ">> Horizontal map wrapping is now OFF.  Generate a maze when ready."
		else
			src.myGrid.grid2d_SetWrap(TRUE,src.myGrid.grid2d_DoesWrapY())
			world	<< ">> Horizontal map wrapping is now ON.  Generate a maze when ready."
	verb/Toggle_Wrap_Y()
		if ( src.myGrid.grid2d_DoesWrapY() )
			src.myGrid.grid2d_SetWrap(src.myGrid.grid2d_DoesWrapX(),FALSE)
			world	<< ">> Vertical map wrapping is now OFF.  Generate a maze when ready."
		else
			src.myGrid.grid2d_SetWrap(src.myGrid.grid2d_DoesWrapX(),TRUE)
			world	<< ">> Vertical map wrapping is now ON.  Generate a maze when ready."



	/*
		Just so you can see some of the data available for each cell, let's
		print some info when one's turf representation is clicked.  Some of this
		is redundant, but I figured the demo will show the different ways to
		ask.
	*/
turf
	Click(location,control,params)
		//	Let's make it shorter to read.  Darn you, contrived variable placement! ;)
		var/dg_Grid/g	= usr.client.myGrid
		//	Handy dandy header
		usr	<< "\icon[src.loc] [src.x],[src.y] \icon[src]"
		//	Cell categories
		usr	<< "\tIsolated? [g.dg_IsIsolated(src.x,src.y) ? "YES" : "NO"]"
		usr	<< "\tFork? [g.dg_IsFork(src.x,src.y) ? "YES" : "NO"]"
		usr	<< "\tFunnel? [g.dg_IsFunnel(src.x,src.y) ? "YES" : "NO"]"
		usr	<< "\tTunnel? [g.dg_IsTunnel(src.x,src.y) ? "YES" : "NO"]"
		usr	<< "\tDead end? [g.dg_IsDeadEnd(src.x,src.y) ? "YES" : "NO"]"
		usr	<< "\tOrigin? [g.dg_IsOrigin(src.x,src.y) ? "YES" : "NO"]"
		var
			list/coords
			list/dirs
			index
		//	Exits
		usr	<< "\t[g.dg_GetNumExits(src.x,src.y)] Exit\s: \..."
		usr	<< "[g.dg_HasExits(src.x,src.y) ? "YES" : "NO"]"
		coords	= g.dg_GetExits(src.x,src.y)
		dirs	= g.dg_GetExitDirs(src.x,src.y)
		for(index = 1 to dirs.len)
			usr	<< "\t\t[g.grid2d_DirToText(dirs[index])] : \..."
			usr	<< "[coords[2*index-1]],[coords[2*index]] \..."
			usr	<< "[g.dg_DoesExitToDir(src.x,src.y,dirs[index]) ? "YES" : "NO"] \..."
			usr	<< "[g.dg_DoesExitToCell(src.x,src.y,coords[2*index-1],coords[2*index]) ? "YES" : "NO"]"
		//	Entrances
		usr	<<	"\t[g.dg_GetNumEntrances(src.x,src.y)] Entrance\s: \..."
		usr	<< "[g.dg_HasEntrances(src.x,src.y) ? "YES" : "NO"]"
		coords	= g.dg_GetEntrances(src.x,src.y)
		dirs	= g.dg_GetEntranceDirs(src.x,src.y)
		for(index = 1 to dirs.len)
			usr	<< "\t\t[g.grid2d_DirToText(dirs[index])] : \..."
			usr	<< "[coords[2*index-1]],[coords[2*index]] \..."
			usr	<< "[g.dg_IsEnteredByDir(src.x,src.y,dirs[index]) ? "YES" : "NO"] \..."
			usr	<< "[g.dg_IsEnteredByCell(src.x,src.y,coords[2*index-1],coords[2*index]) ? "YES" : "NO"]"



















/*
	You can ignore everything below this line.  It's just setting the camera
	and demo atoms.
*/
////////////////////////////////////////////////////////////////////////////////

client
	perspective				= EDGE_PERSPECTIVE|EYE_PERSPECTIVE
	New()
		..()
		src.view	= "[world.maxx]x[world.maxy]"
		src.eye		= locate(world.maxx/2,world.maxy/2,1)

mob
	Login()
		..()
		src.client.eye		= locate(world.maxx/2,world.maxy/2,src.z)
turf
	icon					= 'dg_demoT.dmi'
	icon_state				= "0"
	mouse_opacity			= 2
area/icon					= 'dg_demoA.dmi'
area/DeadEnd/icon_state		= "deadend"
area/Fork/icon_state		= "fork"
area/Isolated/icon_state	= "isolated"
area/Tunnel/icon_state		= "tunnel"
